/****************************************************************************/
/** syringe.cpp                                                            **/
/** ---------------------------------------------------------------------- **/
/** Injects custom dll into specified process                              **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1998-99, OK Thinking Software                            **/
/** ---------------------------------------------------------------------- **/
/** This library is free software; you can redistribute it and/or          **/
/** modify it under the terms of the GNU Lesser General Public             **/
/** License as published by the Free Software Foundation; either           **/
/** version 2 of the License, or (at your option) any later version.       **/
/**                                                                        **/
/** This library is distributed in the hope that it will be useful,        **/
/** but WITHOUT ANY WARRANTY; without even the implied warranty of         **/
/** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      **/
/** Lesser General Public License for more details.                        **/
/**                                                                        **/
/** You should have received a copy of the GNU Lesser General Public       **/
/** License along with this library; if not, write to the Free Software    **/
/** Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA **/
/** ---------------------------------------------------------------------- **/
/**                                  Oleg Kagan (oleg_kagan@geocities.com) **/
/** ---------------------------------------------------------------------- **/
/** History:           (see history.txt file for complete project history) **/
/**                                                                        **/
/** August 13, 1999 - Version 1.0.7.0                                      **/
/**                                  Oleg Kagan (oleg_kagan@geocities.com) **/
/**                                                                        **/
/** August 13, 1999 - Version 1.0.7.1                                      **/
/**   1. SYRINGE_ANSI_FROM_UNICODE_STR macro changed:                      **/
/**     a. Exceptions processing added (because alloca throws an exception **/
/**        in case of failure).                                            **/
/**     b. The success condition now is (szTrg != NULL).                   **/
/**                                  Oleg Kagan (oleg_kagan@geocities.com) **/
/**                                                                        **/
/** August 29, 1999 - Version 1.0.7.2                                      **/
/**   1. Some of "#ifdef WIN32" statements changed to                      **/
/**      "#ifdef _SYRINGE_WIN9X_COMPATIBLE_".                              **/
/**                                  Oleg Kagan (oleg_kagan@geocities.com) **/
/****************************************************************************/

// Switch all optimizations off 
// (Visual C specific... For any other compiler do the same)
//============================================================================
#ifdef _MSC_VER
	#pragma optimize("", off)
#endif // defined(_MSC_VER)

//============================================================================
#include <windows.h>
#pragma hdrstop
#include <malloc.h>

//============================================================================
#include "syringex.h"

#ifdef WIN64
#include <psapi.h>
#pragma comment ( lib, "psapi.lib" ) 
#endif // WIN64

// Creates zero-terminated (char*) string named by szTrg on the stack with 
// ANSI equivalent of c_wszSrc. c_wszSrc suppose to be (const wchar_t*) 
// zero-terminated UNICODE string
//============================================================================
#define SYRINGE_ANSI_FROM_UNICODE_STR(szTrg, c_wszSrc) \
	char* szTrg; \
	try { \
		size_t nSizeW = (size_t) wcslen(c_wszSrc) + (size_t)1; \
		size_t nSizeA = nSizeW * (size_t)2; \
		szTrg = (char*)alloca(nSizeA + (size_t)1); \
		if (szTrg) { \
			ZeroMemory(szTrg, nSizeA + (size_t)1); \
			if ( \
				!WideCharToMultiByte( \
					CP_ACP, 0, \
					c_wszSrc, (int) nSizeW, \
					szTrg, (int)nSizeA, NULL, NULL \
				) /*!WideCharToMultiByte*/ \
			) szTrg = NULL; \
		} /*if*/ \
	} /*try*/ \
	catch (...) {szTrg = NULL;}

// TInjectTools implementation
//============================================================================
const char* const TInjectTools::c_szKernelModule = "Kernel32";
const char* const TInjectTools::c_szLoadLibraryA = "LoadLibraryA";
const char* const TInjectTools::c_szFreeLibrary = "FreeLibrary";
const char* const TInjectTools::c_szGetProcAddressA = "GetProcAddress";
const char* const TInjectTools::c_szGetLastError = "GetLastError";
const char* const TInjectTools::c_szOutputDebugStringA = "OutputDebugStringA";

//----------------------------------------------------------------------------
const char* const TInjectTools::c_szLoadCustomDll = "LoadCustomDll";

//----------------------------------------------------------------------------
const char* const TInjectTools::c_szSyringeGetLastError = (
	"SyringeGetLastError"
) /*TInjectTools::c_szSyringeGetLastError*/;
const char* const TInjectTools::c_szSyringeResetLastError = (
	"SyringeResetLastError"
) /*TInjectTools::c_szSyringeResetLastError*/;
const char* const TInjectTools::c_szInterceptDllCall = "InterceptDllCallA";
const char* const TInjectTools::c_szIsWinNt = "IsWinNt";
const char* const TInjectTools::c_szInjectModule = "InjectModuleA";
const char* const TInjectTools::c_szEnumModules = "EnumModules";

//----------------------------------------------------------------------------
TInjectTools::TInjectTools(

	const char* c_szInitModuleToInject, 
	const char* c_szInitEntryPoint,
	void* pInitParameter,
	size_t nInitParameterSize

){
	memset(this, 0, sizeof(*this));

	GetModuleFileNameA(
		hSyringeInstance, szSyringeFullName, sizeof(szSyringeFullName) - 1
	) /*GetModuleFileNameA*/;
	szSyringeFullName[sizeof(szSyringeFullName) - 1] = char(0);

	hSyringe = NULL;

	strncpy(
		szModuleToInject, c_szInitModuleToInject, sizeof(szModuleToInject) - 1
	) /*strncpy*/;
	szModuleToInject[sizeof(szModuleToInject) - 1] = char(0);

	strncpy(szEntryPoint, c_szInitEntryPoint, sizeof(szEntryPoint) - 1);
	szEntryPoint[sizeof(szEntryPoint) - 1] = char(0);

	hModuleToInject = NULL;
	p_fnEntryPoint = NULL;
	pResult = NULL;

	hTargetModule = NULL;  

	p_fnInterceptDllCall = NULL;
	p_fnIsWinNt = NULL;

	HINSTANCE hKernel = GetModuleHandleA(c_szKernelModule);
	if (hKernel) {
		p_fnLoadLibraryA = TLoadLibraryAPtr(
			GetProcAddress(hKernel, c_szLoadLibraryA)
		) /*p_fnLoadLibraryA*/;

		p_fnFreeLibrary = TFreeLibraryPtr(
			GetProcAddress(hKernel, c_szFreeLibrary)
		) /*p_fnFreeLibrary*/;

		p_fnGetProcAddressA = TGetProcAddressAPtr(
			GetProcAddress(hKernel, c_szGetProcAddressA)
		) /*p_fnGetProcAddressA*/;

		p_fnGetLastError = TGetLastErrorPtr(
			GetProcAddress(hKernel, c_szGetLastError)
		) /*p_fnGetLastError*/;

		p_fnOutputDebugStringA = TOutputDebugStringAPtr(
			GetProcAddress(hKernel, c_szOutputDebugStringA)
		) /*p_fnOutputDebugStringA*/;
	} /*if*/

	strncpy(szLoadCustomDll, c_szLoadCustomDll, sizeof(szLoadCustomDll) - 1);
	szLoadCustomDll[sizeof(szLoadCustomDll) - 1] = char(0);

	pParameter = pInitParameter;
	nParameterSize = nInitParameterSize;
} /*TInjectTools(const char*, const char*, void*, size_t)*/

// SyringeResetLastError()
//============================================================================
DWORD dwSyringeLastError = 0;

//----------------------------------------------------------------------------
DWORD WINAPI SyringeGetLastError() {return dwSyringeLastError;}

//----------------------------------------------------------------------------
DWORD WINAPI SyringeResetLastError()
{
	DWORD dwErrorOrg = dwSyringeLastError;
	dwSyringeLastError = 0;
	return dwErrorOrg;
} /*SyringeResetLastError()*/

// IsWinNt()
//============================================================================
BOOL WINAPI IsWinNt()
{
	static unsigned uIsNt = 2;
	if (uIsNt > 1) {
		// Check NT or Win95/98
		//----------------------
		OSVERSIONINFO VersionInfo; {
			VersionInfo.dwOSVersionInfoSize = sizeof(VersionInfo);
			GetVersionEx(&VersionInfo);
		} /*VersionInfo*/

		uIsNt = (VersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT);
	} /*if*/
	return uIsNt != 0;
} /*Win32IsNt()*/

// TNtUtils
//============================================================================
class TNtUtils {
public:
	// TVirtualAllocExPtr
	//------------------------------------------------------------------------
	typedef LPVOID (WINAPI *TVirtualAllocExPtr)(
		HANDLE, LPVOID, DWORD, DWORD, DWORD
	) /*TVirtualAllocExPtr*/;

	// TVirtualFreeExPtr
	//------------------------------------------------------------------------
	typedef BOOL (WINAPI *TVirtualFreeExPtr)(
		HANDLE, LPVOID, DWORD, DWORD
	) /*TVirtualFreeExPtr*/;

	//------------------------------------------------------------------------
	TVirtualAllocExPtr p_fnVirtualAllocEx;
	TVirtualFreeExPtr p_fnVirtualFreeEx;

	//------------------------------------------------------------------------
	TNtUtils();
} /*TNtUtils*/;

// TNtUtils implementation
//============================================================================
TNtUtils::TNtUtils()
{
	HMODULE hKernel32 = GetModuleHandle("kernel32.dll");
	if (!hKernel32) {
		dwSyringeLastError = GetLastError(); throw TXInjectorFailure();	
	} /*if*/

	p_fnVirtualAllocEx = TVirtualAllocExPtr(
		GetProcAddress(hKernel32, "VirtualAllocEx")
	) /*p_fnVirtualAllocEx*/;
	if (!p_fnVirtualAllocEx) {
		dwSyringeLastError = GetLastError(); throw TXInjectorFailure();	
	} /*if*/

	p_fnVirtualFreeEx = TVirtualFreeExPtr(
		GetProcAddress(hKernel32, "VirtualFreeEx")
	) /*p_fnVirtualFreeEx*/;
	if (!p_fnVirtualFreeEx) {
		dwSyringeLastError = GetLastError(); throw TXInjectorFailure();	
	} /*if*/
} /*TNtUtils()*/

// InterceptDllCallA(...)
//============================================================================
#define MakePtr(Type, Base, Offset) ((Type)(PBYTE(Base) + (DWORD)(Offset)))

//----------------------------------------------------------------------------

#if _MSC_VER <= 1200
#define IFUNC PDWORD
#else
#ifdef WIN64
#define IFUNC ULONGLONG
#else
#define IFUNC DWORD
#endif
#endif

//----------------------------------------------------------------------------

BOOL WINAPI InterceptDllCallA(

	HMODULE hLocalModule, 
	const char* c_szDllName,
	const char* c_szApiName,
	void* pApiNew,
	void** p_pApiOrg,
	void* pApiToChange
	
){
	if (
		!(
			(c_szDllName && *c_szDllName && c_szApiName && *c_szApiName) || 
			pApiToChange
		) /* ||
		IsBadReadPtr(hLocalModule, sizeof(PIMAGE_NT_HEADERS))*/
	) return FALSE;

    IFUNC p_dwAddrToIntercept; 

	if (pApiToChange) {
		p_dwAddrToIntercept = (IFUNC)pApiToChange;
	}
	else {
		p_dwAddrToIntercept = (IFUNC)GetProcAddress(
			GetModuleHandle((char*)c_szDllName), (char*)c_szApiName
		) /*GetProcAddress*/;
	} /*iff*/;

	BOOL bSuccess = FALSE;

    PIMAGE_DOS_HEADER pDOSHeader = PIMAGE_DOS_HEADER(hLocalModule);
	if (pDOSHeader->e_magic != IMAGE_DOS_SIGNATURE) return FALSE;

    PIMAGE_NT_HEADERS pNTHeader = MakePtr(
		PIMAGE_NT_HEADERS, pDOSHeader, pDOSHeader->e_lfanew
	) /*pNTHeader*/;
    if (pNTHeader->Signature != IMAGE_NT_SIGNATURE) return FALSE;
    
    PIMAGE_IMPORT_DESCRIPTOR pImportDesc = MakePtr(
		PIMAGE_IMPORT_DESCRIPTOR, hLocalModule, 
        pNTHeader->OptionalHeader.DataDirectory[
			IMAGE_DIRECTORY_ENTRY_IMPORT
		] /*pNTHeader->OptionalHeader.DataDirectory*/.VirtualAddress
	) /*pImportDesc*/;
    if (pImportDesc == PIMAGE_IMPORT_DESCRIPTOR(pNTHeader)) return FALSE;
    
	while (pImportDesc->Name) 
	{
		PIMAGE_THUNK_DATA pThunk = MakePtr( PIMAGE_THUNK_DATA, hLocalModule, pImportDesc->FirstThunk) /*pThunk*/;
    
		while (pThunk->u1.Function) 
		{
			if ( (IFUNC)p_dwAddrToIntercept == (IFUNC) pThunk->u1.Function )
			{	
/*				if ( !IsBadWritePtr( (LPVOID)(&pThunk->u1.Function), sizeof(DWORD) ))
				{
					if (p_pApiOrg) *p_pApiOrg = PVOID(pThunk->u1.Function);
					pThunk->u1.Function = (IFUNC)(pApiNew);
					bSuccess = TRUE;
				}
				else 
*/				{
					DWORD dwProtect;
					if (VirtualProtect(&(pThunk->u1.Function), sizeof(DWORD),PAGE_EXECUTE_READWRITE, &dwProtect)) /*VirtualProtect*/
					{
						DWORD dwNewProtect;

						if (p_pApiOrg) 
						{
							*p_pApiOrg = PVOID(pThunk->u1.Function);
						} /*if*/
						pThunk->u1.Function = (IFUNC)pApiNew;
						bSuccess = TRUE;

						dwNewProtect = dwProtect;
						VirtualProtect(&(pThunk->u1.Function), sizeof(DWORD),dwNewProtect, &dwProtect) /*VirtualProtect*/;
					} /*if*/
				} /*iff*/
			} /*if*/
			++pThunk;
		} /*while (pThunk->u1.Function)*/
		++pImportDesc;
	} /*while (pImportDesc->Name)*/

    return bSuccess;
} /*InterceptDllCallA(HMODULE, const char*, const char*, void*,void**,void*)*/

// InterceptDllCallW(...)
//============================================================================
BOOL WINAPI InterceptDllCallW(

	HMODULE hLocalModule, 
	const wchar_t* c_wszDllName,
	const wchar_t* c_wszApiName,
	void* pApiNew,
	void** p_pApiOrg,
	void* pApiToChange
	
){
	if (!(c_wszDllName && *c_wszDllName && c_wszApiName && *c_wszApiName))
		return FALSE;

	SYRINGE_ANSI_FROM_UNICODE_STR(szDllName, c_wszDllName);
	if (!szDllName) return FALSE;

	SYRINGE_ANSI_FROM_UNICODE_STR(szApiName, c_wszApiName);
	if (!szApiName) return FALSE;

	return InterceptDllCallA(
		hLocalModule, szDllName, szApiName, pApiNew, p_pApiOrg, pApiToChange
	) /*return InterceptDllCallA*/;
} /*InterceptDllCallW(HMODULE, const wchar_t*, const wchar_t*, void*, ...)*/

// InjectModuleA(...)
//============================================================================
//----------------------------------------------------------------------------
void InjectModuleCleanup(

	HANDLE hProcess,
	void*& r_pRemoteMemory,
	HANDLE& r_hRemoteThread

){
	if (r_hRemoteThread) {
		CloseHandle(r_hRemoteThread); r_hRemoteThread = NULL;
	} /*if*/
	if (r_pRemoteMemory) {
		TNtUtils NtUtils;
		NtUtils.p_fnVirtualFreeEx(hProcess, r_pRemoteMemory, 0, MEM_RELEASE);
		r_pRemoteMemory = NULL;
	} /*if*/
} /*InjectModuleCleanup()*/

  // InjectorThread(...)
  //============================================================================
static DWORD WINAPI InjectorThread(

	LPVOID lpParameter

	) {
	if (!lpParameter) return FALSE;
	TInjectTools& r_InjectTools = *(TInjectTools*)lpParameter;

	if (
		!r_InjectTools.p_fnLoadLibraryA || !r_InjectTools.p_fnFreeLibrary ||
		!r_InjectTools.p_fnGetProcAddressA ||
		!r_InjectTools.p_fnGetLastError || !r_InjectTools.p_fnOutputDebugStringA
		) return FALSE;

	// Load syringe.dll 
	//------------------
	r_InjectTools.hSyringe = r_InjectTools.p_fnLoadLibraryA(
		r_InjectTools.szSyringeFullName
		) /*r_InjectTools.hSyringe*/;
	if (!r_InjectTools.hSyringe) {
		r_InjectTools.dwLastError = r_InjectTools.p_fnGetLastError();
		return FALSE;
	} /*if*/

	  // Load and "execute" custom DLL
	  //-------------------------------
	TLoadCustomDllPtr p_fnLoadCustomDllPtr = TLoadCustomDllPtr(
		r_InjectTools.p_fnGetProcAddressA(
			r_InjectTools.hSyringe, r_InjectTools.szLoadCustomDll
			) /*r_InjectTools.p_fnGetProcAddressA*/
		) /*p_fnLoadCustomDllPtr*/;
	if (!p_fnLoadCustomDllPtr) {
		r_InjectTools.dwLastError = r_InjectTools.p_fnGetLastError();
		return FALSE;
	} /*if*/

	BOOL bResult = p_fnLoadCustomDllPtr(r_InjectTools);

	r_InjectTools.p_fnFreeLibrary(r_InjectTools.hSyringe);
	r_InjectTools.hSyringe = NULL;

	return bResult;
} /*InjectorThread(LPVOID)*/
static void InjectorThreadEnd() {}
	//----------------------------------------------------------------------------
BOOL WINAPI InjectModuleA(

	HANDLE hProcess, 
	HANDLE,
	const char* c_szModuleToInject,
	const char* c_szEntryPoint,
	void* pParameter, 
	size_t nParameterSize,
	void** p_pResult,
	DWORD dwTimeOutMSec

){
	if (
		!hProcess ||
		!c_szModuleToInject || !*c_szModuleToInject ||
		!c_szEntryPoint || !*c_szEntryPoint
	) return FALSE;

	if (!pParameter || !nParameterSize) {
		pParameter = NULL;
		nParameterSize = 0;
	} /*if*/

	// Adjust timeout
	//----------------
	if (!dwTimeOutMSec) dwTimeOutMSec = INFINITE;
	if ((dwTimeOutMSec != INFINITE) && (dwTimeOutMSec < 100)) 
		dwTimeOutMSec = 100;

	size_t nInjectorCodeSize = (
		(char*)&InjectorThreadEnd - (char*)&InjectorThread
	) /*nInjectorCodeSize*/;

	size_t nInjectToolsSize = sizeof(TInjectTools);

	size_t nInjectorSize = (
		nInjectorCodeSize + nInjectToolsSize + nParameterSize + 1024
	) /*nInjectorSize*/;

	TNtUtils NtUtils;
	void* pRemoteInjector = NtUtils.p_fnVirtualAllocEx(
		hProcess, NULL, (DWORD) nInjectorSize, MEM_COMMIT, PAGE_EXECUTE_READWRITE
	) /*NtUtils.p_fnVirtualAllocEx*/;
	if (!pRemoteInjector) {
		dwSyringeLastError = GetLastError(); return FALSE;
	} /*if*/

	void* pRemoteInjectTools = (
		((char*)pRemoteInjector) + nInjectorCodeSize + 1
	) /*pRemoteInjectTools*/;
	void* pRemoteParameter = (
		((char*)pRemoteInjectTools) + nInjectToolsSize + 1
	) /*pRemoteParameter*/;

	TInjectTools InjectTools(
		c_szModuleToInject, c_szEntryPoint, pRemoteParameter, nParameterSize
	) /*InjectTools*/;

	DWORD dwInjectorThreadId = 0;
	HANDLE hInjectorThread = NULL;

	try {
		// Transfer code of the injector
		//-------------------------------
		{
			SIZE_T dwBytesWritten;
			if (	
				!WriteProcessMemory(
					hProcess, pRemoteInjector, 
					&InjectorThread, nInjectorCodeSize, &dwBytesWritten
				) /*!WriteProcessMemory*/
			){
				dwSyringeLastError = GetLastError(); 
				throw TXInjectorFailure();
			} /*if*/
		} /*Transfer code of the injector*/

		// Transfer injection tools
		//--------------------------
		{
			SIZE_T dwBytesWritten;
			if (
				!WriteProcessMemory(
					hProcess, pRemoteInjectTools, 
					&InjectTools, nInjectToolsSize, &dwBytesWritten
				) /*!WriteProcessMemory*/
			){
				dwSyringeLastError = GetLastError(); 
				throw TXInjectorFailure();
			} /*if*/
		} /*Transfer injection tools*/

		// Transfer parameter
		//--------------------
		if (pParameter) {
			SIZE_T dwBytesWritten;
			if (
				!WriteProcessMemory(
					hProcess, pRemoteParameter, 
					pParameter, nParameterSize, &dwBytesWritten
				) /*!WriteProcessMemory*/
			){
				dwSyringeLastError = GetLastError(); 
				throw TXInjectorFailure();
			} /*if*/
		} /*if*/

		// Run injector
		//--------------
		{
			hInjectorThread = CreateRemoteThread(
				hProcess, NULL, 0, 
				(LPTHREAD_START_ROUTINE)pRemoteInjector, pRemoteInjectTools,  
				0, &dwInjectorThreadId
			) /*CreateRemoteThread*/;
			if (!hInjectorThread) {
				dwSyringeLastError = GetLastError(); 
				throw TXInjectorFailure();
			} /*if*/

			DWORD dwWaitResult = WaitForSingleObject(
				hInjectorThread, dwTimeOutMSec
			) /*WaitForSingleObject*/;

			switch (dwWaitResult) {
				case WAIT_FAILED: 
					dwSyringeLastError = GetLastError(); 
					throw TXInjectorFailure();

				case WAIT_TIMEOUT: 
					dwSyringeLastError = dwWaitResult; 
					TerminateThread(hInjectorThread, FALSE);
					throw TXInjectorFailure();
			} /*switch (dwWaitResult)*/
		} /*Run injector*/

		// Read results of the execution
		//-------------------------------
		{
			SIZE_T dwBytesReaded;
			if (
				!ReadProcessMemory(
					hProcess, pRemoteInjectTools, 
					&InjectTools, nInjectToolsSize, &dwBytesReaded
				) /*!ReadProcessMemory*/ 
			){
				dwSyringeLastError = GetLastError(); 
				throw TXInjectorFailure();
			} /*if*/

			if (pParameter) {
				if (
					!ReadProcessMemory(
						hProcess, pRemoteParameter, 
						pParameter, nParameterSize, &dwBytesReaded
					) /*!ReadProcessMemory*/ 
				){
					dwSyringeLastError = GetLastError(); 
					throw TXInjectorFailure();
				} /*if*/
			} /*if*/
		} /*Read results of the execution*/

		if (p_pResult) *p_pResult = InjectTools.pResult;
		
		DWORD dwInjectorThreadExitCode = 0;
		if (!GetExitCodeThread(hInjectorThread, &dwInjectorThreadExitCode)) {
			dwSyringeLastError = GetLastError(); 
			throw TXInjectorFailure();
		} /*if*/

		if (!dwInjectorThreadExitCode) {
			dwSyringeLastError = InjectTools.dwLastError; 
			throw TXInjectorFailure();
		} /*if*/
	} /*try*/
	catch (TXInjectorFailure&) {
		InjectModuleCleanup(hProcess, pRemoteInjector, hInjectorThread);
		return FALSE;
	} /*catch (TXInjectionFailure&)*/
	catch (...) {
		InjectModuleCleanup(hProcess, pRemoteInjector, hInjectorThread);
		throw;
	} /*catch (...)*/

	InjectModuleCleanup(hProcess, pRemoteInjector, hInjectorThread);

	return TRUE;
} /*InjectModuleA(HANDLE, HANDLE, const char*, const char*, void*, ...)*/

// InjectModuleW(...)
//============================================================================
BOOL WINAPI InjectModuleW(

	HANDLE hProcess, 
	HANDLE hInstrumentalThread,  // ignored under Windows NT
	const wchar_t* c_wszModuleToInject, const wchar_t* c_wszEntryPoint,
	void* pParameter, size_t nParameterSize,
	void** p_pResult,
	DWORD dwTimeOutMSec

){
	if (
		!hProcess ||
		!c_wszModuleToInject || !*c_wszModuleToInject ||
		!c_wszEntryPoint || !*c_wszEntryPoint
	) return FALSE;

	SYRINGE_ANSI_FROM_UNICODE_STR(szModuleToInject, c_wszModuleToInject);
	if (!szModuleToInject) return FALSE;

	SYRINGE_ANSI_FROM_UNICODE_STR(szEntryPoint, c_wszEntryPoint);
	if (!szEntryPoint) return FALSE;

	return InjectModuleA(
		hProcess, hInstrumentalThread, szModuleToInject, szEntryPoint,
		pParameter, nParameterSize, p_pResult, dwTimeOutMSec
	) /*return InjectModuleA*/;
} /*InjectModuleW(HANDLE, HANDLE, const wchar_t*, const wchar_t*, void*,...)*/

// EnumModules(...)
//============================================================================
BOOL WINAPI EnumModules(

	HMODULE* a_hModules,     // array to receive the module handles
	size_t nSizeInBytes,     // size of the array in bytes
	size_t* p_nBytesWritten  // receives the number of bytes returned
	
){
	HMODULE hMainModule = GetModuleHandle(NULL);
		
	size_t nSize = nSizeInBytes / sizeof(HMODULE);
	size_t nWritten = 0;

	BYTE* pRegion = NULL;
	MEMORY_BASIC_INFORMATION Mbi;
	memset ( &Mbi, 0, sizeof ( Mbi ) ) ;
	char szModule[2];
	size_t lQuerySize = 0 ;

	while ( (nWritten < nSize) &&
		    ( lQuerySize = VirtualQuery(pRegion, &Mbi, sizeof(Mbi)) ) == sizeof(Mbi)
	){
		if (Mbi.State == MEM_FREE) Mbi.AllocationBase = Mbi.BaseAddress;
		if (
			Mbi.AllocationBase &&
			(Mbi.AllocationBase != hMainModule) &&
			(Mbi.AllocationBase == Mbi.BaseAddress) &&
			GetModuleFileNameA(
				HMODULE(Mbi.AllocationBase), szModule, sizeof(szModule)
			) /*GetModuleFileNameA*/
		) a_hModules[nWritten++] = HMODULE(Mbi.AllocationBase);
		pRegion += Mbi.RegionSize;
	} /*while*/

#ifdef WIN64
	if ( nWritten ==  0 )
	{
		DWORD lSize = 0 ;
		EnumProcessModules ( GetCurrentProcess (), a_hModules, (DWORD) nSizeInBytes, &lSize ) ;
		nWritten = lSize ;
	}
#endif // WIN64

	if ( p_nBytesWritten ) 
		*p_nBytesWritten = nWritten * sizeof ( HMODULE ) ;

	return nWritten != 0;
} /*EnumModules(HMODULE*, size_t, size_t*)*/

// DllEntryPoint(...)
//============================================================================
HINSTANCE hSyringeInstance = NULL;

//----------------------------------------------------------------------------
extern "C" BOOL WINAPI DllEntryPoint(

	HINSTANCE hDllInstance,  // handle to DLL module 
	DWORD dwReason,  // reason for calling function 
	LPVOID   // reserved 
 
){
	try {
		switch (dwReason) {
			case DLL_PROCESS_ATTACH: {
				hSyringeInstance = hDllInstance;
				SyringeResetLastError();
#ifdef NEVER
				char szModule[MAX_PATH + 1];
				GetModuleFileName(NULL, szModule, sizeof(szModule) - 1);
				MessageBox(
					NULL, szModule, "Attach syringe.dll", 
					MB_OK | MB_SYSTEMMODAL
				) /*MessageBox*/;
#endif // NEVER
			} /*case DLL_PROCESS_ATTACH*/ break;

			case DLL_PROCESS_DETACH: {
#ifdef NEVER
				char szModule[MAX_PATH + 1];
				GetModuleFileName(NULL, szModule, sizeof(szModule) - 1);
				MessageBox(
					NULL, szModule, "Detach syringe.dll", 
					MB_OK | MB_SYSTEMMODAL
				) /*MessageBox*/;
#endif // NEVER
			} /*case DLL_PROCESS_DETACH*/ break;

			case DLL_THREAD_ATTACH: {
			} /*case DLL_THREAD_ATTACH*/ break;

			case DLL_THREAD_DETACH: {
			} /*case DLL_THREAD_DETACH*/ break;
		} /*switch (dwReason)*/
	} /*try*/
	catch (...) {
		return false;
	} /*catch (...)*/

	return true;
} /*DllEntryPoint(HINSTANCE, DWORD, LPVOID)*/

// LoadCustomDll(...)
//============================================================================
extern "C" BOOL WINAPI LoadCustomDll(

	TInjectTools& r_InjectTools

){
	// Init some of the r_InjectTools fields
	//---------------------------------------
	r_InjectTools.hTargetModule = GetModuleHandleA(NULL);

	r_InjectTools.p_fnSyringeGetLastError = TSyringeGetLastErrorPtr(
		GetProcAddress(
			r_InjectTools.hSyringe, TInjectTools::c_szSyringeGetLastError
		) /*GetProcAddress*/
	) /*r_InjectTools.p_fnSyringeGetLastError*/;
	if (!r_InjectTools.p_fnSyringeGetLastError) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/

	r_InjectTools.p_fnSyringeResetLastError = TSyringeResetLastErrorPtr(
		GetProcAddress(
			r_InjectTools.hSyringe, TInjectTools::c_szSyringeResetLastError
		) /*GetProcAddress*/
	) /*r_InjectTools.p_fnSyringeResetLastError*/;
	if (!r_InjectTools.p_fnSyringeResetLastError) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/

	r_InjectTools.p_fnIsWinNt = TIsWinNtPtr(
		GetProcAddress(r_InjectTools.hSyringe, TInjectTools::c_szIsWinNt)
	) /*r_InjectTools.p_fnIsWinNt*/;
	if (!r_InjectTools.p_fnIsWinNt) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/

	r_InjectTools.p_fnInterceptDllCall = TInterceptDllCallPtr(
		GetProcAddress(
			r_InjectTools.hSyringe, TInjectTools::c_szInterceptDllCall
		) /*GetProcAddress*/
	) /*r_InjectTools.p_fnInterceptDllCall*/;
	if (!r_InjectTools.p_fnInterceptDllCall) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/

	r_InjectTools.p_fnInjectModule = TInjectModulePtr(
		GetProcAddress(
			r_InjectTools.hSyringe, TInjectTools::c_szInjectModule
		) /*GetProcAddress*/
	) /*r_InjectTools.p_fnInjectModule*/;
	if (!r_InjectTools.p_fnInjectModule) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/

	r_InjectTools.p_fnEnumModules = TEnumModulesPtr(
		GetProcAddress(r_InjectTools.hSyringe, TInjectTools::c_szEnumModules)
	) /*r_InjectTools.p_fnEnumModules*/;
	if (!r_InjectTools.p_fnEnumModules) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/
	
	// Load custom DLL
	//-----------------
	r_InjectTools.hModuleToInject = LoadLibraryA(
		r_InjectTools.szModuleToInject
	) /*LoadLibraryA*/;
	if (!r_InjectTools.hModuleToInject) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/

	// Get and execute the "entry point"
	//-----------------------------------
	r_InjectTools.p_fnEntryPoint = TSyringeDllEntryPointPtr(
		GetProcAddress(
			r_InjectTools.hModuleToInject, r_InjectTools.szEntryPoint
		) /*GetProcAddress*/
	) /*r_InjectTools.p_fnEntryPoint*/;
	if (!r_InjectTools.p_fnEntryPoint) {
		r_InjectTools.dwLastError = GetLastError(); return FALSE;
	} /*if*/

	r_InjectTools.pResult = r_InjectTools.p_fnEntryPoint(&r_InjectTools);

	FreeLibrary(r_InjectTools.hModuleToInject);

	return TRUE;
} /*LoadCustomDll(TInjectTools&)*/

// Switch off all stack checks
// (Visual C specific... For any other compiler do the same thing)
//============================================================================
#ifdef _MSC_VER
	#pragma check_stack (off)
#endif // defined(_MSC_VER)

// Switch all stack checks to the original state
// (Visual C specific... For any other compiler do the same thing)
//============================================================================
#ifdef _MSC_VER
	#pragma check_stack
#endif // defined(_MSC_VER)
