#ifndef __SYRINGEX_H__
#define __SYRINGEX_H__
/****************************************************************************/
/** syringex.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Internal SYRINGE.DLL tools declaration                                 **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1998-99, OK Thinking Software                            **/
/** ---------------------------------------------------------------------- **/
/** This library is free software; you can redistribute it and/or          **/
/** modify it under the terms of the GNU Lesser General Public             **/
/** License as published by the Free Software Foundation; either           **/
/** version 2 of the License, or (at your option) any later version.       **/
/**                                                                        **/
/** This library is distributed in the hope that it will be useful,        **/
/** but WITHOUT ANY WARRANTY; without even the implied warranty of         **/
/** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      **/
/** Lesser General Public License for more details.                        **/
/**                                                                        **/
/** You should have received a copy of the GNU Lesser General Public       **/
/** License along with this library; if not, write to the Free Software    **/
/** Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA **/
/** ---------------------------------------------------------------------- **/
/**                                  Oleg Kagan (oleg_kagan@geocities.com) **/
/** ---------------------------------------------------------------------- **/
/** History:           (see history.txt file for complete project history) **/
/**                                                                        **/
/** August 13, 1999 - Version 1.0.7.0                                      **/
/**                                                                        **/
/** August 29, 1999 - Version 1.0.7.2                                      **/
/**   1. _SYRINGE_WIN9X_COMPATIBLE_ - definition have been added to enable **/
/**      Win9x compatibility for i386 compatible target.                   **/
/**                                  Oleg Kagan (oleg_kagan@geocities.com) **/
/****************************************************************************/

//============================================================================
#ifndef _SYRINGE_DECLSPEC_
	#define _SYRINGE_DECLSPEC_ _SYRINGE_EXPORT_
#endif /*!defined(_SYRINGE_DECLSPEC_)*/

//============================================================================
#include "syringe.h"

// Use the next definition to enable Win9x compatibility. 
// NOTE: Valid for i386 compatible processors only !!!
//       Define it using compiler option in Your project or makefile, please.
//============================================================================
#ifdef _SYRINGE_WIN9X_COMPATIBLE_
	#pragma message("Win9x compatibility enabled...")
#else
	#pragma message("Win9x compatibility disabled...")
#endif // defined(_SYRINGE_WIN9X_COMPATIBLE_)

//============================================================================
extern HINSTANCE hSyringeInstance;
extern DWORD dwSyringeLastError;

// TXInjectorFailure
// Default exception class
//============================================================================
struct TXInjectorFailure {};	

// TInjectTools
// provides basic function pointers (enough to load module and execute the 
// main function) to the remote thread 
//============================================================================
struct TInjectTools : public TInjectInfo {
public:
	//------------------------------------------------------------------------
	static const char* const c_szKernelModule;
	static const char* const c_szLoadLibraryA;
	static const char* const c_szFreeLibrary;
	static const char* const c_szGetProcAddressA;
	static const char* const c_szGetLastError;
	static const char* const c_szOutputDebugStringA;

	//------------------------------------------------------------------------
	static const char* const c_szLoadCustomDll;

	//------------------------------------------------------------------------
	static const char* const c_szSyringeGetLastError;
	static const char* const c_szSyringeResetLastError;
	static const char* const c_szInterceptDllCall;
	static const char* const c_szIsWinNt;
	static const char* const c_szInjectModule;
	static const char* const c_szEnumModules;

	//------------------------------------------------------------------------
	typedef HINSTANCE (WINAPI *TLoadLibraryAPtr)(LPCTSTR);
	typedef BOOL (WINAPI *TFreeLibraryPtr)(HMODULE);
	typedef FARPROC (WINAPI *TGetProcAddressAPtr)(HMODULE, LPCSTR);
	typedef HMODULE (WINAPI *TGetModuleHandleAPtr)(LPCTSTR);
	typedef DWORD (WINAPI *TGetLastErrorPtr)(VOID);
	typedef int (WINAPI *TOutputDebugStringAPtr)(LPCTSTR);

	//------------------------------------------------------------------------
	TInjectTools(
		const char* c_szInitModuleToInject, const char* c_szInitEntryPoint,
		void* pInitParameter, size_t nInitParameterSize
	) /*TInjectTools(const char*, const char*, void*, size_t)*/;

	//------------------------------------------------------------------------
	TLoadLibraryAPtr p_fnLoadLibraryA;
	TFreeLibraryPtr p_fnFreeLibrary;
	TGetProcAddressAPtr p_fnGetProcAddressA;
	TGetLastErrorPtr p_fnGetLastError;
	TOutputDebugStringAPtr p_fnOutputDebugStringA;

	//------------------------------------------------------------------------
	DWORD dwLastError;
	char szLoadCustomDll[32];
	char a_cDummy[16];
} /*TInjectTools*/;

//============================================================================
typedef BOOL (WINAPI *TLoadCustomDllPtr)(TInjectTools&);

#endif /*!defined(__SYRINGEX_H__)*/


